library("RMVL")

Matlas<-mvl_open("O3a_atlas.mvl")

Mout<-mvl_open("O3a_spatial_index.mvl", append=TRUE, create=TRUE)

cat("Generating auxiliary data\n")

mvl_block_write<-function(MVLHANDLE, N, f, name=NULL, block=1000000, type=NULL, ...) {
	obj<-NULL
	offset<-NULL
	for(i in seq(1.0, N, by=block)) {
		idx<-i-1+(1:block)
		idx<-idx[idx<=N]
		
		v<-f(idx, ...)
		
		if(is.null(obj)) {
			if(!is.null(type))attr(v, "MVL_TYPE")<-type
			offset<-mvl_start_write_vector(MVLHANDLE, v, expected.length=N, name=name)
			MVLHANDLE<-mvl_remap(MVLHANDLE)
			obj<-MVLHANDLE[offset,ref=TRUE]
			} else {
			mvl_rewrite_vector(obj, i, v)
			}
		}
	return(invisible(offset))
	}

cat("\tfrequency band start\n")
mvl_block_write(Mout, length(Matlas$skymaps[,"idx"]), function(idx){ return(Matlas$parameters[Matlas$skymaps[idx, "idx"],"band_start"]) }, "skymaps_band_start", type=4)

cat("\tx0\n")
mvl_block_write(Mout, length(Matlas$skymaps[,"dec"]), function(idx){ return(cos(Matlas$skymaps[idx,"ra"])*cos(Matlas$skymaps[idx,"dec"])) }, "x0", type=4)

cat("\tx1\n")
mvl_block_write(Mout, length(Matlas$skymaps[,"dec"]), function(idx){ return(sin(Matlas$skymaps[idx,"ra"])*cos(Matlas$skymaps[idx,"dec"])) }, "x1", type=4)

cat("\tx2\n")
mvl_block_write(Mout, length(Matlas$skymaps[,"dec"]), function(idx){ return(sin(Matlas$skymaps[idx,"dec"])) }, "x2", type=4)


Mout<-mvl_remap(Mout)

cat("Writing index\n")
mvl_write_spatial_index1(Mout, list(Mout$skymaps_band_start, Mout$x0, Mout$x1, Mout$x2), c(12, 7, 7, 7), "spatial_index")

mvl_close(Mout)



