library("RMVL")

#
# We find all points which were analyzed by the second stage of the pipeline
#


#
# And we exclude frequency bands that have outlier count a factor of 3 larger than median
#
MaxCountMultiple<-3

Matlas<-mvl_open("O3a_atlas.mvl")

Mstage<-Matlas$skymaps[,"stage"]

N<-dim(Matlas$skymaps)[1]

Nchunk<- 16e6

L<-list()

for(i in seq(1, N, Nchunk)) {
	cat(".")
	idx<-i-1+(1:Nchunk)
	idx<-idx[idx<=N]
	
	Foutlier<-Mstage[idx]==2
	if(any(Foutlier)) {
		L[[length(L)+1]]<-idx[Foutlier]
		}
	}
cat("\n")

outlier_idx<-do.call(c, L)

cat("Found", length(outlier_idx), "points analyzed with second stage of the pipeline\n")

cat("Creating outlier table\n")
Mtmp<-mvl_open("full_scan2_tmp.mvl", append=TRUE, create=TRUE)

# Create MVL data frame with all the found outliers

mvl_indexed_copy(Mtmp, Matlas["skymaps", ref=TRUE], outlier_idx, "all_outliers1")

Mtmp<-mvl_remap(Mtmp)

# Filter outliers by excluding bands which produce too many

mvl_merge(Mtmp, Mtmp["all_outliers1", ref=TRUE], Matlas["parameters", ref=TRUE], "all_outliers2", by="idx", only.columns.y=c("band_start"))

Mtmp<-mvl_remap(Mtmp)


Groups<-mvl_group(list(Mtmp$all_outliers2[,"band_start",ref=TRUE]))

LCounts<-mvl_group_lapply(Groups, function(x) { return(data.frame(band_start=Mtmp$all_outliers2[x[[1]], "band_start"], count=length(x))) })

Counts<-do.call(rbind, LCounts)

FilteredCounts<-Counts[Counts[,"count"]<MaxCountMultiple*median(Counts[,"count"]), ]

cat("Filtered out", 100*(1.0-sum(FilteredCounts[,"count"])/sum(Counts[,"count"])), "% of outliers by excluding", 100*(1-dim(FilteredCounts)[1]/dim(Counts)[1]), "% of frequency bands\n")

filtered_idx<-(1:(dim(Mtmp$all_outliers2)[1]))[Mtmp$all_outliers2[,"band_start", recurse=TRUE] %in% FilteredCounts[,"band_start"]]

# Write out results in MVL and CSV formats

Mout<-mvl_open("full_scan2_filtered_outliers.mvl", create=TRUE, append=TRUE)

mvl_indexed_copy(Mout, Mtmp["all_outliers2", ref=TRUE], filtered_idx, "filtered_outliers")

Mout<-mvl_remap(Mout)

outliers<-Mout$filtered_outliers[,]

write.table(outliers, "full_scan2_filtered_outliers.csv", col.names=TRUE, row.names=FALSE, sep="\t")

mvl_close(Mout)
mvl_close(Mtmp)







